import { MarkdownCell, RawCell } from '@jupyterlab/cells';
import { OutputAreaModel, SimplifiedOutputArea } from '@jupyterlab/outputarea';
import { createNotebookContext, createNotebookPanel } from './create_notebook_panel';
import { GridStackItem } from './item';
export const VIEW = 'grid_default';
export class AppModel {
    constructor(options) {
        this.options = options;
        this._isDisposed = false;
        this._notebook = options.notebook;
    }
    /**
     * Whether the handler is disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    dispose() {
        var _a, _b;
        if (this.isDisposed) {
            return;
        }
        this._isDisposed = true;
        (_a = this._context) === null || _a === void 0 ? void 0 : _a.dispose();
        (_b = this._notebookPanel) === null || _b === void 0 ? void 0 : _b.dispose();
    }
    get notebook() {
        return this._notebook;
    }
    get rendermime() {
        return this.options.rendermime;
    }
    get cells() {
        var _a;
        return (_a = this._context) === null || _a === void 0 ? void 0 : _a.model.cells;
    }
    get context() {
        return this._context;
    }
    get panel() {
        return this._notebookPanel;
    }
    async initialize() {
        this._context = await createNotebookContext({
            manager: this.options.manager
        });
        this._context.model.fromJSON(this.options.notebook);
        this._notebookPanel = createNotebookPanel({
            context: this._context,
            rendermime: this.options.rendermime,
            editorServices: this.options.editorServices
        });
        // TODO Shameless hack, need to do better!
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this.options.tracker.widgetAdded.emit(this._notebookPanel);
    }
    _getCellInfo(id) {
        var _a;
        if (!this.cells) {
            return undefined;
        }
        for (let i = 0; i < this.cells.length; i++) {
            const cell = this.cells.get(i);
            if (cell.id === id) {
                const data = cell.sharedModel.getMetadata().extensions;
                return (_a = data === null || data === void 0 ? void 0 : data.jupyter_dashboards) === null || _a === void 0 ? void 0 : _a.views[VIEW];
            }
        }
        return undefined;
    }
    createCell(cellModel) {
        let item;
        const info = this._getCellInfo(cellModel.id);
        switch (cellModel.type) {
            case 'code': {
                const outputareamodel = new OutputAreaModel({ trusted: true });
                const out = new SimplifiedOutputArea({
                    model: outputareamodel,
                    rendermime: this.options.rendermime
                });
                item = new GridStackItem(cellModel.id, out, info);
                break;
            }
            case 'markdown': {
                const markdownCell = new MarkdownCell({
                    model: cellModel,
                    rendermime: this.options.rendermime,
                    contentFactory: this.options.contentFactory,
                    editorConfig: this.options.editorConfig.markdown
                });
                markdownCell.inputHidden = false;
                markdownCell.rendered = true;
                Private.removeElements(markdownCell.node, 'jp-Collapser');
                Private.removeElements(markdownCell.node, 'jp-InputPrompt');
                item = new GridStackItem(cellModel.id, markdownCell, info);
                break;
            }
            default: {
                const rawCell = new RawCell({
                    model: cellModel,
                    contentFactory: this.options.contentFactory,
                    editorConfig: this.options.editorConfig.raw
                });
                rawCell.inputHidden = false;
                Private.removeElements(rawCell.node, 'jp-Collapser');
                Private.removeElements(rawCell.node, 'jp-InputPrompt');
                item = new GridStackItem(cellModel.id, rawCell, info);
                break;
            }
        }
        return item;
    }
    async executeCell(cell, output) {
        if (cell.type !== 'code' || !this._context) {
            return;
        }
        const source = cell.sharedModel.source;
        const rep = await SimplifiedOutputArea.execute(source, output, this._context.sessionContext);
        return rep;
    }
}
var Private;
(function (Private) {
    /**
     * Remove children by className from an HTMLElement.
     */
    function removeElements(node, className) {
        const elements = node.getElementsByClassName(className);
        for (let i = 0; i < elements.length; i++) {
            elements[i].remove();
        }
    }
    Private.removeElements = removeElements;
})(Private || (Private = {}));
