import { PromiseDelegate } from '@lumino/coreutils';
import { MessageLoop } from '@lumino/messaging';
import { Panel, Widget } from '@lumino/widgets';
import { GridStack } from 'gridstack';
export class AppWidget extends Panel {
    constructor(options) {
        super();
        this._ready = new PromiseDelegate();
        this._gridElements = [];
        this.node.id = options.id;
        this.title.label = options.label;
        this.title.closable = true;
        this._model = options.model;
        this._gridHost = document.createElement('div');
        this._gridHost.className = 'grid-stack';
        this._gridHost.classList.add('jp-App-Launcher-gridhost');
        this.node.appendChild(this._gridHost);
        this._spinner = document.createElement('div');
        this._spinner.classList.add('jp-al-Spinner');
        const spinner = document.createElement('div');
        spinner.className = 'jp-al-SpinnerContent';
        this._spinner.appendChild(spinner);
        this.node.appendChild(this._spinner);
        this.node.style.overflow = 'auto';
        this._grid = GridStack.init({
            float: true,
            column: 12,
            margin: 3,
            cellHeight: 40,
            styleInHead: true,
            disableOneColumnMode: true
        }, this._gridHost);
        this._model.initialize().then(() => {
            this.render()
                .catch(console.error)
                .then(() => window.dispatchEvent(new Event('resize')));
        });
    }
    /**
     * A promise that is fulfilled when the model is ready.
     */
    get ready() {
        return this._ready.promise;
    }
    get model() {
        return this._model;
    }
    get gridWidgets() {
        return this._gridElements;
    }
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._model.dispose();
        super.dispose();
    }
    addGridItem(out) {
        this._gridElements.push(out);
        const info = out.info;
        const id = out.cellIdentity;
        if (info && info.hidden) {
            return;
        }
        const options = {
            id,
            autoPosition: true,
            noMove: true,
            noResize: true,
            locked: true,
            w: 12,
            h: 4
        };
        if (info && info.row !== null && info.col !== null) {
            options.x = info.col;
            options.y = info.row;
            options.w = info.width;
            options.h = info.height;
            options.autoPosition = false;
        }
        MessageLoop.sendMessage(out, Widget.Msg.BeforeAttach);
        this._grid.addWidget(out.node, options);
        MessageLoop.sendMessage(out, Widget.Msg.AfterAttach);
    }
    updateGridItem(id, info) {
        const items = this._grid.getGridItems();
        const item = items === null || items === void 0 ? void 0 : items.find(value => { var _a; return ((_a = value.gridstackNode) === null || _a === void 0 ? void 0 : _a.id) === id; });
        if (!item) {
            return;
        }
        let option;
        if (!info) {
            option = { autoPosition: true };
        }
        else {
            option = {
                x: info.col,
                y: info.row,
                w: info.width,
                h: info.height,
                noMove: true,
                noResize: true,
                locked: true
            };
        }
        this._grid.update(item, option);
    }
    async render() {
        var _a;
        const cellList = (_a = this._model.cells) !== null && _a !== void 0 ? _a : [];
        for (const cell of cellList) {
            const src = cell.sharedModel.source;
            if (src.length === 0) {
                continue;
            }
            const el = this._model.createCell(cell);
            const ret = await this._model.executeCell(cell, el.cellOutput);
            const outputNode = el.cellOutput.node;
            if (outputNode.childNodes.length > 0) {
                this.addGridItem(el);
                if (ret && ret.content.status === 'error') {
                    this.updateGridItem(el.cellIdentity, { width: 12, height: 4 });
                }
            }
        }
        this.node.removeChild(this._spinner);
    }
    onCloseRequest(msg) {
        this._model.dispose();
        super.onCloseRequest(msg);
    }
}
